@@ -169,13 +169,17 @@ Attribute       Description
         >>> c.type
         'grid_mapping'
 
+        >>> c = cf.CoordinateReference('my_new_type', crtype='formula_terms')
+        >>> c.type
+        'formula_terms'
+
         >>> c = cf.CoordinateReference('my_new_type')
         >>> print c.type
         None
 
-        >>> c = cf.CoordinateReference('my_new_type', crtype='formula_terms')
-        >>> c.type
-        'formula_terms'
+        >>> c = cf.CoordinateReference('my_new_type', crtype='grid_mapping')
+        >>> print c.type
+        'grid_mapping'
 
     coords : sequence of str, optional
         Identify the dimension and auxiliary coordinate objects which
@@ -244,13 +248,19 @@ Attribute       Description
             crtype = t
         #--- End: if
 
+#        standard_coords = {}
+#        for coord in _coordinates.get(name, ()):
+#            standard_coords[coord] = coord
+
         if coords is None:
-            coords = set(_coordinates.get(name, ()))
+            coords = set(_coordinates.get(name, ()))          
+#            coords = set(standard_coords)
         else:
             coords = set(coords)
 
         if coord_terms:
-            coords.update([kwargs[term] for term in coord_terms if term in kwargs])
+            coords.update([kwargs[term]
+                           for term in coord_terms if term in kwargs])
             coord_terms = set(coord_terms)
         else:
             coord_terms = set()
@@ -259,6 +269,8 @@ Attribute       Description
         self.name        = name 
         self.coords      = coords
         self.coord_terms = coord_terms
+
+#        self._standard_coords = standard_coords 
     #--- End: def
 
     def __delitem__(self, item):
@@ -383,6 +395,24 @@ Called by def match
         return matches
     #--- End: def
 
+    # ----------------------------------------------------------------
+    # Attribute (read only)
+    # ----------------------------------------------------------------
+    @property
+    def hasbounds(self):
+        '''
+
+False
+
+:Examples:
+
+>>> c.hasbounds
+False
+
+'''
+        return False
+    #--- End: def
+
     @classmethod
     def canonical_units(cls, term):
         '''Return the canonical units for a standard CF coordinate conversion
@@ -416,7 +446,7 @@ None
     def T(self):
         '''
 
-Returns False.
+False.
 
 Provides compatibility with the `cf.Coordinate` API.
 
@@ -516,21 +546,31 @@ Close all files referenced by coordinate conversion term values.
                 value.close()
     #--- End: def
 
-    def copy(self):
+    def copy(self, domain=None):
         '''
 
 Return a deep copy.
 
 ``c.copy()`` is equivalent to ``copy.deepcopy(c)``.
 
+:Examples 1:
+
+>>> d = c.copy()
+
+:Parameters:
+
+    domain : cf.Domain, optional
+
 :Returns:
 
     out :
         The deep copy.
 
-:Examples:
+:Examples 2:
 
->>> d = c.copy()
+>>> domain
+<CF Domain: (110, 106, 3, 5)>
+>>> d = c.copy(domain=domain)
 
 '''       
         X = type(self)
@@ -538,10 +578,11 @@ Return a deep copy.
 
         coord_terms = self.coord_terms.copy()
 
-        new.coord_terms = coord_terms
-        new.coords      = self.coords.copy()
-        new.name        = self.name
-        new.type        = self.type
+        new.coord_terms      = coord_terms
+        new.coords           = self.coords.copy()
+#        new._standard_coords = self._standard_coords.copy()
+        new.name             = self.name
+        new.type             = self.type
 
         _dict = {}
         for term, value in self._dict.iteritems():
@@ -553,6 +594,20 @@ Return a deep copy.
         #--- End: for
         new._dict = _dict
 
+        if domain is not None:
+            # --------------------------------------------------------
+            # Replace coordinate identifiers with coordinate names
+            # derived from a domain object
+            # --------------------------------------------------------
+            coord_map = {}
+            for name in new.coords:
+                coord = domain.item(name, role=('d', 'a',), exact=True)
+                if coord is not None:
+                    coord_map[name] = coord.name(default=None, identity=True)
+            #--- End: for
+            new.change_coord_identities(coord_map, i=True)
+        #--- End: if
+
         return new
     #---End: def
 
@@ -587,7 +642,7 @@ None
         return _default_values.get(term, None)
     #--- End: def
 
-    def dump(self, complete=False, display=True, level=0, domain=None):
+    def dump(self, complete=False, display=True, _level=0, domain=None):
         '''
 
 Return a string containing a full description of the coordinate
@@ -602,8 +657,6 @@ reference.
         the description is printed, i.e. ``c.dump()`` is equivalent to
         ``print c.dump(display=False)``.
 
-    level : int, optional
-
     domain : cf.Domain, optional
 
 :Returns:
@@ -614,8 +667,8 @@ reference.
 :Examples:
 
         '''          
-        indent0 = '    ' * level
-        indent1 = '    ' * (level+1)
+        indent0 = '    ' * _level
+        indent1 = '    ' * (_level+1)
 
         try:
             string = ['%sCoordinate reference: %s' % (indent0, self.identity(''))]
@@ -628,24 +681,26 @@ reference.
                 if value in coord_keys:
                     string.append("%s%s = %r" % (indent1, key, domain.get(value)))
                 else:
+                    # value is a field
                     if complete and hasattr(value, 'domain'):
                         string.append(
                             "%s%s = \n%s" % 
                             (indent1, key, 
                              value.dump(
                                  complete=False, display=False, 
-                                 level=level+2,
-                                 title='Coordinate reference field', q='-')))
+                                 _level=_level+2,
+                                 _title='Coordinate reference field', _q='-')))
                     else:
                         string.append("%s%s = %r" % (indent1, key, value))
         else:
-            for key, value in sorted(self.iteritems()):
+             for key, value in sorted(self.iteritems()):
                 if complete and hasattr(value, 'domain'):
+                    # value is a field
                     string.append("%s%s = \n%s" % 
                                   (indent1, key, 
                                    value.dump(complete=False, display=False, 
-                                              level=level+2, 
-                                              title='Coordinate reference field', q='-')))
+                                              _level=_level+2, 
+                                              _title='Coordinate reference field', _q='-')))
                 else:
                     string.append("%s%s = %r" % (indent1, key, value))
         #--- End: if
@@ -1055,21 +1110,19 @@ conditions.
             return not bool(inverse)
     #--- End: def
 
-    def change_coord(self, value, new=None):
-        '''Change a link to a dimension or auxiliary coordinate object.
+    def change_coord_identities(self, coord_map, i=False):
+        '''Change the idientifier of all coordinates.
 
-If no new value is specified the coordinate object is unlinked from
-the coordinate reference.
+If a coordinate identifier is not in the provided mapping then it is
+set to `None` and thus effectively removed from the coordinate
+reference.
 
 :Parameters:
 
-    value : str
-        The existing identifier to a linked dimension or auxiliary
-        coordinate object.
+    coord_map : dict
+        For example: ``{'dim2': 'dim3', 'aux2': 'latitude', 'aux4': None}``
 
-    new : str, optional
-        The new identifier for the coordinate. By default the
-        coordinate object is unlinked from the coordinate reference.
+    i : bool, optional
 
 :Returns:
 
@@ -1077,37 +1130,65 @@ the coordinate reference.
 
 :Examples:
 
->>> c = cf.CoordinateReference('atmosphere_hybrid_height_coordinate',
+>>> r = cf.CoordinateReference('atmosphere_hybrid_height_coordinate',
 ...                             coord_terms=['a', 'b'],
 ...                             a='ncvar:ak',
 ...                             b='ncvar:bk')
->>> c.coords
+>>> r.coords
 {'atmosphere_hybrid_height_coordinate', 'ncvar:ak', 'ncvar:bk'}
->>> c.items()
-[('a', 'ncvar:ak'), ('b', 'ncvar:bk')]
->>> c.change_coord('ncvar:ak', 'long_name:ak')
->>> c.coords
-{'atmosphere_hybrid_height_coordinate', 'long_name:ak', 'ncvar:bk'}
->>> c.items()
-[('a', 'long_name:ak'), ('b', 'ncvar:bk')]
+>>> r.change_coord_identitiers({'atmosphere_hybrid_height_coordinate', 'dim1',
+...                             'ncvar:ak': 'aux0'})
+>>> r.coords
+{'dim1', 'aux0'}
 
         '''
-        if value == new:
-            return
+        if i:
+            r = self
+        else:
+            r = self.copy()
 
-        for term in self.coord_terms.copy():
-            if self[term] == value:
-                self[term] = new
-        #---End: for
+#        if keep_unmapped:
+        if not coord_map:
+            return r
 
-        coords = self.coords
-        if value in coords:
-            coords.remove(value)
-            if new is not None:
-                coords.add(new)
-        #--- End: if
+#        coord_map = coord_map.copy()
+#        for name in r.coords:
+#            if name not in coord_map:
+#                coord_map[name] = name
+#        #--- End: for
+
+        for term in self.coord_terms:
+            coord = r[term]
+            r[term] = coord_map.get(coord, coord)
+
+#        coords = set([coord_map.get(coord, None) for coord in r.coords])
+        coords = set([coord_map.get(coord, coord) for coord in r.coords])
+        coords.discard(None)
+        r.coords = coords
+
+#        _standard_coords = [(coord_map.get(name, name), sn) 
+#                            for name, sn in r._standard_coords.iteritems()]
+#        r._standard_coords = dict(_standard_coords)
+
+        return r
     #---End: def
 
+#    def reset_coords(self, *identifiers, i=False):
+#        coord_map = {}
+#        for identifier in identifiers:
+#            coord_map[identifier] = self._reset_coords.get(identifier, None)
+#
+#        return self.change_coord_identities(coord_map, i=i)
+#    #--- End: def
+
+#    def reset_coord(self, identifier, i=False):
+#        '''
+##        '''
+#        new_name = self._standard_coords.get(identifier, None)    
+#        coord_map = {identifier: new_name}
+#        return self.change_coord_identities(coord_map, i=i)
+#    #--- End: def
+
     def remove_all_coords(self):
         '''Remove all links to coordinate objects.
 
