@@ -1,8 +1,10 @@
 import atexit
 
+from numpy import array       as numpy_array
 from numpy import dtype       as numpy_dtype
 from numpy import expand_dims as numpy_expand_dims
 from numpy import ndarray     as numpy_ndarray
+from numpy import number      as numpy_number
 from numpy import transpose   as numpy_transpose
 from numpy import vectorize   as numpy_vectorize
 
@@ -693,6 +695,11 @@ otherwise a memory leak could occur. For example:
                 p_data = subarray.view()
         #--- End: if
 
+        if not p_data.ndim and isinstance(p_data, numpy_number):
+            # p_data is a numby number (like numpy.int64) which does
+            # not support assignment, so convert to a numpy array.
+            p_data = numpy_array(p_data)
+
         masked = numpy_ma_isMA(p_data)
         if masked:
             # --------------------------------------------------------
