@@ -36,11 +36,10 @@ from .functions import _close_netcdf_file, _open_netcdf_file
 
 def write(fields, filename, fmt='NETCDF3_CLASSIC', overwrite=True,
           verbose=False, cfa_options=None, mode='w',
-          least_significant_digit=None, endian='native', 
-          compress=False, complevel=4, fletcher32=False, no_shuffle=False):
-    '''
-    
-Write fields to a CF-netCDF or CFA-netCDF file.
+          least_significant_digit=None, endian='native',
+          compress=0, fletcher32=False, no_shuffle=False,
+          datatype=None, single=False, double=False):
+    '''Write fields to a CF-netCDF or CFA-netCDF file.
     
 NetCDF dimension and variable names will be taken from variables'
 `~Variable.ncvar` attributes and the domain attribute
@@ -146,7 +145,32 @@ and auxiliary coordinate roles for different data variables.
            =======  ==================================================
        
         By default the file is opened with write access mode ``'w'``.
+
+    datatype : dict, optional
+        Specify data type conversions to be applied prior to writing
+        data to disk. Arrays with data types which are not specified
+        remain unchanged. By default, array data types are preserved.
+
+          **Example:**
+            To convert 64 bit floats and integers to their 32 bit
+            counterparts: ``datatype={numpy.dtype(float):
+            numpy.dtype('float32'), numpy.dtype(int):
+            numpy.dtype('int32')}``.
        
+    single : bool, optional
+        Write 64-bit floats as 32-bit floats and 64-bit integers as
+        32-bit integers. By default, input data types are
+        preserved. Note that ``single=True`` is equivalent to
+        ``datatype={numpy.dtype('float64'): numpy.dtype('float32'),
+        numpy.dtype('int64'): numpy.dtype('int32')}``.
+       
+    double : bool, optional
+        Write 32-bit floats as 64-bit floats and 32-bit integers as
+        64-bit integers. By default, input data types are
+        preserved. Note that ``double=True`` is equivalent to
+        ``datatype={numpy.dtype('float32'): numpy.dtype('float64'),
+        numpy.dtype('int32'): numpy.dtype('int64')}``.
+ 
 :Returns:
 
     None
@@ -179,17 +203,42 @@ and auxiliary coordinate roles for different data variables.
  <CF Field: v_compnt_of_wind(19, 29, 24)>,
  <CF Field: potential_temperature(19, 30, 24)>]
 
-'''  
+    ''' 
+    compress = int(compress)
+    zlib = bool(compress)
+
     if fmt not in ('NETCDF3_CLASSIC', 'NETCDF3_64BIT', 'CFA3',
                    'NETCDF4', 'NETCDF4_CLASSIC', 'CFA4'):
-        raise ValueError("Unknown output file format: %s" % fmt)
+        raise ValueError("Unknown output file format: {0}".format(fmt))
 
     if compress and fmt in ('NETCDF3_CLASSIC', 'NETCDF3_64BIT', 'CFA3'):
-        raise ValueError("Can't compress %s format file" % fmt)
+        raise ValueError("Can't compress {0} format file".format(fmt))
     
     if least_significant_digit and fmt in ('CFA3', 'CFA4'):
-        raise ValueError("Can't truncate data variables in %s format file" % fmt)
-    
+        raise ValueError("Can't truncate data variables in {0} format file".format(fmt))
+
+    # ----------------------------------------------------------------
+    # Set up data type conversions
+    # ----------------------------------------------------------------
+    dtype_conversions = {numpy_dtype(bool)  : numpy_dtype('int32'),
+                         numpy_dtype(object): numpy_dtype(float)}
+    if datatype:
+        if single:
+            raise ValueError("Can't set datatype and single")
+        if double:
+            raise ValueError("Can't set datatype and double")
+        dtype_conversions.update(datatype)
+    else:
+        if single and double:
+            raise ValueError("Can't set single and double")
+        if single:
+            dtype_conversions[numpy_dtype(float)] = numpy_dtype('float32')
+            dtype_conversions[numpy_dtype(int)]   = numpy_dtype('int32')
+        if double:
+            dtype_conversions[numpy_dtype('float32')] = numpy_dtype(float)
+            dtype_conversions[numpy_dtype('int32')]   = numpy_dtype(int)
+    datatype = dtype_conversions
+
     # ----------------------------------------------------------------
     # Initialize dictionary of useful global variables
     # ----------------------------------------------------------------
@@ -242,8 +291,8 @@ and auxiliary coordinate roles for different data variables.
          # -----------------------------------------------------------
          # Compression/endian
          # -----------------------------------------------------------
-         'compression'            : {'zlib'       : compress,
-                                     'complevel'  : complevel,
+         'compression'            : {'zlib'       : zlib,
+                                     'complevel'  : compress,
                                      'fletcher32' : fletcher32,
                                      'shuffle'    : not no_shuffle},
          'endian'                 : endian,
@@ -254,7 +303,13 @@ and auxiliary coordinate roles for different data variables.
          # -----------------------------------------------------------
          'omit_bounds_properties': ('units', 'standard_name', 'axis',
                                     'positive', 'calendar', 'month_lengths',
-                                    'leap_year', 'leap_month')
+                                    'leap_year', 'leap_month'),
+
+         'least_significant_digit': least_significant_digit,
+         # ------------------------------------------------------------
+         # Specify data type conversions to be applied prior to writing
+         # ------------------------------------------------------------
+         'datatype'               : datatype,
          }
 
     if fmt == 'CFA3':
@@ -277,22 +332,22 @@ and auxiliary coordinate roles for different data variables.
     # ---------------------------------------------------------------
     # Still here? Open the output netCDF file.
     # ---------------------------------------------------------------
-    if mode != 'w':
-        raise ValueError("Can only set mode='w' at the moment")
+#    if mode != 'w':
+#        raise ValueError("Can only set mode='w' at the moment")
 
     filename = os_path_expanduser(os_path_expandvars(filename))
 
-    if isfile(filename):
+    if mode == 'w' and isfile(filename):
         if not overwrite:
             raise IOError(
-                "Can't write to an existing file unless overwrite=True: %s" %
-                abspath(filename))
-
+                "Can't write to an existing file unless overwrite=True: {0}".format(
+                    abspath(filename)))
+                
         if not os.access(filename, os.W_OK):
             raise IOError(
-                "Can't overwrite an existing file without permission: %s" %
-                abspath(filename))
-
+                "Can't overwrite an existing file without permission: {0}".format(
+                    abspath(filename)))
+            
         _close_netcdf_file(filename)
         remove(filename)
     #--- End: if          
@@ -454,7 +509,7 @@ trailing dimension.
     return new
 #--- End: def
 
-def _datatype(variable):
+def _datatype(variable, g=None):
     '''
 
 Return the netCDF4.createVariable datatype corresponding to the
@@ -478,6 +533,8 @@ If the input variable has no `!dtype` attribute (or it is None) then
         Any object with a `!dtype` attribute whose value is a
         `numpy.dtype` object or None.
 
+    g : dict
+
 :Returns:
 
     out : str
@@ -490,17 +547,15 @@ If the input variable has no `!dtype` attribute (or it is None) then
         variable.dtype is None):
         return 'S1'            
 
-    elif variable.dtype is numpy_dtype(bool):
-        # Boolean arrays will be written as 8-bit signed integers (0
-        # or 1)
-        return 'i1'
+    dtype = variable.dtype
 
-    elif variable.dtype is numpy_dtype(object):
-        # Object arrays will be written as 64-bit floats
-        return 'f8'
-    
-    # Still here?
-    return '%s%s' % (variable.dtype.kind, variable.dtype.itemsize)
+    convert_dtype = g['datatype']
+
+    new_dtype = convert_dtype.get(dtype, None)
+    if new_dtype is not None:
+        dtype = new_dtype
+        
+    return '{0}{1}'.format(dtype.kind, dtype.itemsize)
 #--- End: def
 
 def _string_length_dimension(size, g=None):
@@ -633,7 +688,7 @@ written to the file.
 '''
     fill_value = data.fill_value # False, None speed?
 
-    g['nc'][ncvar] = g['netcdf'].createVariable(ncvar, _datatype(data), (),
+    g['nc'][ncvar] = g['netcdf'].createVariable(ncvar, _datatype(data, g=g), (),
                                                 fill_value=fill_value,
                                                 least_significant_digit=None,
                                                 endian=g['endian'],
@@ -676,6 +731,8 @@ written to the file.
     if base is not None:
         cfa_array['base'] = base
 
+    convert_dtype = g['datatype']
+
     for attrs in cfa_array['Partitions']:
         fmt = attrs.get('format', None)
 
@@ -694,10 +751,10 @@ written to the file.
 
             array = partition.dataarray(**pda_args)
 
-            if array.dtype is numpy_dtype(bool):
-                # Write a boolean array as an array of 8-bit signed
-                # integers (0 or 1)
-                array = array.astype('int32')
+            # Convert data type
+            new_dtype = convert_dtype.get(array.dtype, None)
+            if new_dtype is not None:
+                array = array.astype(new_dtype)  
                 
             shape = array.shape        
             ncdim_strlen = []
@@ -722,7 +779,7 @@ written to the file.
             cfa_dimensions = [_cfa_dimension(n, g=g) for n in array.shape]
             
             # Create the private CFA variable and write the array to it
-            v = g['netcdf'].createVariable(p_ncvar, _datatype(array),
+            v = g['netcdf'].createVariable(p_ncvar, _datatype(array, g=g),
                                            cfa_dimensions + ncdim_strlen,
                                            fill_value=fill_value,
                                            least_significant_digit=None,
@@ -730,6 +787,7 @@ written to the file.
                                            **g['compression'])
             
             _write_attributes(v, {'cf_role': 'cfa_private'})
+            
             v[...] = array
 
             # Update the attrs dictionary.
@@ -1512,9 +1570,8 @@ Write a grid mapping georeference to the netCDF file.
         grid_mapping.append(ncvar)
 #--- End: def
 
-def _create_netcdf_variable(ncvar, dimensions, cfvar, omit=(), extra={},
-                            data_variable=False,
-                            g=None):
+def _create_netcdf_variable(ncvar, dimensions, cfvar, omit=(),
+                            extra={}, data_variable=False, g=None):
     '''
 
 Create a netCDF variable from *cfvar* with name *ncvar* and dimensions
@@ -1551,7 +1608,7 @@ created. The ``seen`` dictionary is updated for *cfvar*.
 
 '''
     # Set the netCDF4.createVariable datatype
-    datatype = _datatype(cfvar)
+    datatype = _datatype(cfvar, g=g)
     ncdimensions = dimensions
     
     if not cfvar._hasData:
@@ -1651,8 +1708,8 @@ created. The ``seen`` dictionary is updated for *cfvar*.
                 **g['compression'])
         except (TypeError, RuntimeError):
             raise TypeError(
-                "Can't write data type {0!r} from {1!r} to a {2} file".format(
-                    datatype, cfvar, g['netcdf'].file_format))
+"Can't write data type {0!r} (from {1!r}) to a {2} file. Use a netCDF4 format or use the single precision option.".format(
+    datatype, cfvar, g['netcdf'].file_format))
 
         _write_attributes(g['nc'][ncvar], netcdf_attrs)
 
@@ -1689,12 +1746,15 @@ created. The ``seen`` dictionary is updated for *cfvar*.
                 pda_args['update'] = False
             #--- End: if
 
+            convert_dtype = g['datatype']
+
             for partition in data.partitions.flat:
                 array = partition.dataarray(**pda_args)   
 
-                # Convert booleans to integers
-                if array.dtype is numpy_dtype(bool):
-                    array = array.astype('int32')                    
+                # Convert data type
+                new_dtype = convert_dtype.get(array.dtype, None)
+                if new_dtype is not None:
+                    array = array.astype(new_dtype)  
 
                 # Check that the array doesn't contain any elements
                 # which are equal to any of the missing data values
@@ -1710,6 +1770,7 @@ created. The ``seen`` dictionary is updated for *cfvar*.
 cfvar)
                 #--- End: if
 
+
                 # Copy the array into the netCDF variable
                 g['nc'][ncvar][partition.indices] = array        
 
