@@ -44,19 +44,28 @@ destination grid that can then be used by the run_regridding method.
         contributes to conservative regridding.
 
     method: string, optional
-        By default the regridding method is conservative. If this
-        parameter is set to 'bilinear' then multilinear interpolation
-        is used. If it is set to 'nearest_stod' then nearest neighbor
+        By default the regridding method is set to 'conservative'. In
+        this case first-order conservative regridding is used. If it
+        is set to 'bilinear' then multilinear interpolation is used.
+        If it is set to 'nearest_stod' then nearest neighbor
         interpolation is used where each destination point is mapped
-        to the closest source point. A given source point may go to
+        to the closest source point. A given source point may map to
         multiple destination points, but no destination point will
         receive input from more than one source point. If it is set
         to 'nearest_dtos' then nearest neighbor interpolation is used
-        where each destination point is mapped to the closest source
-        point. A given source point may go to multiple destination
-        points, but no destination point will receive input from more
-        than one source point.
-    
+        where each source point is mapped to the closest destination
+        point. A given destination point may receive input from
+        multiple source points, but no source point will map to more
+        than one destination point.
+
+    srchasbounds: bool, optional
+        Whether or not the source field has bounds for use by the
+        auto method.
+
+    dsthasbounds: bool, optional
+        Whether or not the destination fields has bounds for use by
+        the auto method
+
         """
         # create a handle to the regridding method
         if method == 'conservative':
@@ -147,14 +156,6 @@ grid may have an associated mask.
 
         """
         if use_bounds:
-            # Check the bounds for contiguity if they exist
-            if lon.hasbounds and not lon.contiguous(overlap=False):
-                raise ValueError('The longitude bounds must be contiguous.')
-            #--- End: if
-            if lat.hasbounds and not lon.contiguous(overlap=False):
-                raise ValueError('The latitude bounds must be contiguous.')
-            #--- End: if
-            
             # Get the bounds, creating them if they do not exist
             x_bounds = lon.get_bounds(create=True)
             y_bounds = lat.get_bounds(create=True).clip(-90, 90, 'degrees').array
@@ -162,7 +163,8 @@ grid may have an associated mask.
             # If not set as cyclic already, check for cyclicity
             if not cyclic:
                 cyclic = abs(x_bounds.datum(-1) - x_bounds.datum(0)) == Data(360,
-                         'degrees')
+                             'degrees')
+            #--- End: if
             x_bounds = x_bounds.array
         #--- End: if
         
@@ -206,7 +208,7 @@ grid may have an associated mask.
         #--- End: if
         
         # Add the mask if appropriate
-        if not mask is None:
+        if mask is not None:
             gmask = grid.add_item(ESMF.GridItem.MASK)
             gmask[...] = 1
             gmask[mask] = 0
@@ -260,9 +262,6 @@ grid may have an associated mask.
         The resulting ESMPy grid for use as a source or destination
         grid in regridding.
 
-    out: bool
-        A boolean indicating whether or not the grid has bounds.
-
         """
         # Get the shape of the grid
         shape = lon.transpose(x_order).shape
@@ -271,15 +270,9 @@ grid may have an associated mask.
                              ' have the same shape.')
         
         # Check whether bounds exist or not, and get them if they do
-        hasbounds = lon.hasbounds and lat.hasbounds and use_bounds
-        if hasbounds:
+        if use_bounds:
             x_bounds = lon.bounds
             y_bounds = lat.bounds.clip(-90, 90, 'degrees')
-            if not x_bounds.contiguous(overlap=False):
-                raise ValueError('The 2D longitude bounds must be contiguous.')
-            if not y_bounds.contiguous(overlap=False):
-                raise ValueError('The 2D latitude bounds must be contiguous.')
-            
             n = x_bounds.shape[0]
             m = x_bounds.shape[1]
             x_bounds = x_bounds.array
@@ -299,17 +292,20 @@ grid may have an associated mask.
 
             x_bounds = tmp_x
             y_bounds = tmp_y
+        #--- End: if
         
         # Create empty grid
         max_index = numpy_array(shape, dtype='int32')
-        if hasbounds:
+        if use_bounds:
             staggerLocs = [ESMF.StaggerLoc.CORNER, ESMF.StaggerLoc.CENTER]
         else:
             staggerLocs = ESMF.StaggerLoc.CENTER
+        #--- End: if
         if cyclic:
             grid = ESMF.Grid(max_index, num_peri_dims=1, staggerloc=staggerLocs)
         else:
             grid = ESMF.Grid(max_index, staggerloc=staggerLocs)
+        #--- End: if
         
         # Populate grid centres
         x, y = 0, 1
@@ -319,7 +315,7 @@ grid may have an associated mask.
         gridYCentre[...] = lat.transpose(y_order).array
         
         # Populate grid corners if there are bounds
-        if hasbounds:
+        if use_bounds:
             gridCorner = grid.coords[ESMF.StaggerLoc.CORNER]
             x_bounds = x_bounds.transpose(x_order)
             y_bounds = y_bounds.transpose(y_order)
@@ -328,15 +324,16 @@ grid may have an associated mask.
                 y_bounds = y_bounds[:-1,:]
             gridCorner[x][...] = x_bounds
             gridCorner[y][...] = y_bounds
+        #--- End: if
         
         # Add the mask if appropriate
-        if not mask is None:
+        if mask is not None:
             gmask = grid.add_item(ESMF.GridItem.MASK)
             gmask[...] = 1
             gmask[mask] = 0
         #--- End: if
         
-        return grid, hasbounds
+        return grid
     #--- End: def
 
     @staticmethod
@@ -425,22 +422,22 @@ of the grid.
                 gridCorner = grid.coords[ESMF.StaggerLoc.CORNER]
             else:
                 gridCorner = grid.coords[ESMF.StaggerLoc.CORNER_VFACE]
+            #--- End: if
             for d in xrange(0, ndim):
-                if coords[d].hasbounds and not coords[d].contiguous(overlap=False):
-                    raise ValueError('Coordinate ' + str(d) +
-                                     ' does not have contiguous bounds.')
-                #--- End: if
                 boundsD = coords[d].get_bounds(create=True).array
                 if shape[d] > 1:
                     tmp = numpy_empty(shape[d] + 1)
                     tmp[0:-1] = boundsD[:, 0]
                     tmp[-1] = boundsD[-1, 1]
                     boundsD = tmp
+                #--- End: if
                 gridCorner[d][...] = boundsD.reshape([shape[d] + 1 if x == d else 1
                                                       for x in xrange(0, ndim)])
+            #--- End: for
+        #--- End: if
         
         # Add the mask if appropriate
-        if not mask is None:
+        if mask is not None:
             gmask = grid.add_item(ESMF.GridItem.MASK)
             gmask[...] = 1
             gmask[mask] = 0
