@@ -3,6 +3,7 @@ import cf
 import os
 import unittest
 import atexit
+import numpy
 
 tmpfile  = tempfile.mktemp('.cf-python_test')
 tmpfiles = [tmpfile]
@@ -50,7 +51,7 @@ class read_writeTest(unittest.TestCase):
         self.assertTrue(len(cf.read(filename, top_level=('field', 'measure', 'coordinate'))) == 24)
     #--- End: def
 
-    def test_read_write(self):
+    def test_read_write_format(self):
         # No compression
         for chunksize in self.chunk_sizes:   
             cf.CHUNKSIZE(chunksize) 
@@ -64,9 +65,11 @@ class read_writeTest(unittest.TestCase):
                 cf.write(f, tmpfile, fmt=fmt)
                 g = cf.read(tmpfile)[0]
                 self.assertTrue(f.equals(g, traceback=True),
-                                'Bad read/write of format: %s' % fmt)
+                                'Bad read/write of format: {0}'.format(fmt))
         #--- End: for
+    #--- End: def
 
+    def test_read_write_netCDF4_compress_shuffle(self):
         # Compression
         for chunksize in self.chunk_sizes:   
             cf.CHUNKSIZE(chunksize) 
@@ -75,19 +78,32 @@ class read_writeTest(unittest.TestCase):
                         'NETCDF4_CLASSIC',
                         'CFA4'):
                 for no_shuffle in (True, False):
-                    for complevel in range(10):
+                    for compress in range(10):
                         cf.write(f, tmpfile, fmt=fmt,
-                                 compress=True, 
-                                 complevel=complevel,
+                                 compress=compress,
                                  no_shuffle=no_shuffle)
                         g = cf.read(tmpfile)[0]
                         self.assertTrue(
                             f.equals(g, traceback=True),
-                            'Bad read/write format with lossless compression: %s' % fmt)
+                            'Bad read/write with lossless compression: {0}, {1}, {2}'.format(fmt, compress, no_shuffle))
         #--- End: for
         cf.CHUNKSIZE(self.original_chunksize) 
     #--- End: def
 
+    def test_write_datatype(self):
+#        if self.test_only and inspect.stack()[0][3] not in self.test_only:
+#            return
+
+        # Compression
+        for chunksize in self.chunk_sizes:   
+            cf.CHUNKSIZE(chunksize) 
+            f = cf.read(self.filename)[0] 
+            self.assertTrue(f.dtype == numpy.dtype(float))
+            cf.write(f, tmpfile, fmt='NETCDF4', 
+                     datatype={numpy.dtype(float): numpy.dtype('float32')})
+            g = cf.read(tmpfile)
+            self.assertTrue(g.dtype == numpy.dtype('float32'), 
+                            'datatype read in is '+str(g.dtype))
 #--- End: class
 
 if __name__ == "__main__":
